$Console
$ScreenHide
_Source _Console
_Dest _Console

Type INT24
    LO As _Unsigned Integer
    HI As _Unsigned _Byte
End Type

Type ZCMHEADER
    PTR As _Unsigned Integer
    Bank As _Unsigned _Byte
    Size As INT24
    VERA_Cfg As _Unsigned _Byte
    VERA_Rate As _Unsigned _Byte
End Type

Dim Shared HEADER As ZCMHEADER
Dim Shared DefRate$
DefRate$ = "11025"
Const Stereo = &B00010000
Const bit16 = &B00100000
Const CFGBASE = &B00001111

start:

CheckParameters
ClearZCM_Header
R_FILE$ = Get_RAWFile$
If Not _FileExists(R_FILE$) Then
    _MessageBox "File Not Found", R_FILE$, "error"
    System
End If

HEADER.VERA_Rate = Get_Vera_Rate(R_FILE$)
HEADER.VERA_Cfg = Audio_Format
On Error GoTo zfErr
ZCFile$ = GetZCMTarget(R_FILE$)

Write_ZCMFile R_FILE$, ZCFile$
System

zfErr:
Resume start

RATEDATA:
Data "8000","11025","16000","22000","22050","44100","48000"

Sub Write_ZCMFile (RAW_FILE$, Z_File$)
    Dim SndByte As _Unsigned _Byte
    StartZWrite:
    If _FileExists(Z_File$) Then
        KillIt% = _MessageBox("FILE EXISTS", Z_File$ + " Exists\nOverwrite?", "yesno", "question", 0)
        If KillIt% = 1 Then
            Kill (Z_File$)
        Else
            Z_File$ = _InputBox$("NEW FILE NAME", "Enter ZCM File Name", Z_File$)
            If Z_File$ = "" Then System
            GoTo StartZWrite
        End If
    End If
    Print "Writing: "; Z_File$;
    Open Z_File$ For Binary As #4
    Open RAW_FILE$ For Binary As #2
    Put #4, , HEADER
    Get #2, , SndByte
    Do
        Put #4, , SndByte
        Increment_Header_Size
        If HEADER.Size.LO Mod 1024 = 0 Then Print ".";
        Get #2, , SndByte
    Loop Until EOF(2)
    PRINT
    Close #2
    Seek #4, 1
    Put #4, , HEADER
    Close #4

    Print "SUCCESS !!  "; Z_File$
    _MessageBox "SUCCESS !!", Z_File$, "info"

End Sub

Sub Increment_Header_Size
    If HEADER.Size.HI = 255 Then
        _MessageBox "FILE TO LARGE", "Max Size is 16mb! ", "error"
        Close #2
        Close #4
        System
    End If
    If HEADER.Size.LO = 65535 Then
        HEADER.Size.HI = HEADER.Size.HI + 1
        HEADER.Size.LO = 0
    Else
        HEADER.Size.LO = HEADER.Size.LO + 1
    End If
End Sub

Function GetZCMTarget$ (SRC$)
    TMP$ = GetFilePath(SRC$) + GetFileName(SRC$) + ".ZCM"
    TMP$ = _SaveFileDialog$("SAVE ZCM FILE", TMP$, "*.ZCM|*.zcm", "ZCM Audio")
    If TMP$ = "" Then System
    GetZCMTarget = TMP$
End Function

Sub ClearZCM_Header
    HEADER.PTR = 0
    HEADER.Bank = 0
    HEADER.Size.LO = 0
    HEADER.Size.HI = 0
    HEADER.VERA_Rate = 0
    HEADER.VERA_Cfg = CFGBASE
End Sub

Function Audio_Format%
    fmt.base = CFGBASE
    is16 = _MessageBox("16 Bit File ?", "Is this a 16 bit RAW file ?", "yesno", "question", 0)
    If is16 = 1 Then fmt.base = fmt.base Or bit16
    IsStereo = _MessageBox("Stereo AUDIO ?", "Is this a stereo RAW file ?", "yesno", "question", 0)
    If IsStereo = 1 Then fmt.base = fmt.base Or Stereo
    Audio_Format = fmt.base
End Function

Function Get_Vera_Rate% (RAW_FILE$)
    Dim TMP As _Unsigned Integer
    CheckDefRate RAW_FILE$
    REDO:
    MESG$ = "Enter the Sample Rate of \n" + RAW_FILE$
    RATE$ = _InputBox$("AUDIO SAMPLE RATE", MESG$, DefRate$)
    If IsNum(RATE$) Then
        TMP = Val(RATE$)
        If TMP > 1 And TMP <= 48828 Then DONE% = -1
    End If
    If DONE% Then
        Get_Vera_Rate = ZCMRATE(TMP)
    Else
        If RATE$ = "" Then System
        _MessageBox "INVALID AUDIO RATE", "TRY AGAIN", "error"
        GoTo REDO
    End If
End Function

Sub CheckDefRate (RAW_FILE$)
    Restore RATEDATA
    For I = 1 To 7
        Read RATE$
        If InStr(1, RAW_FILE$, RATE$) <> 0 Then
            DefRate$ = RATE$
            Exit Sub
        End If
    Next I
End Sub

Function Get_RAWFile$
    FName$ = _OpenFileDialog$("Select Raw Audio File", "", "*.raw|*.RAW", "Raw Audio", 0)
    If FName$ = "" Then System
    Get_RAWFile$ = FName$
End Function

Sub CheckParameters
    Rem
End Sub

Function ZCMRATE% (TRUERATE As _Unsigned Integer)
    ZCMRATE = _Round((TRUERATE / 48828) * 128)
End Function

Function GetFileExt$ (FName$)
    DOT$ = "."
    L = Len(FName$)
    TMP$ = ""
    If InStr(1, FName$, DOT$) = 0 Then GoTo Skipall
    For I = L To 1 Step -1
        C$ = Mid$(FName$, I, 1)
        TMP$ = C$ + TMP$
        If C$ = DOT$ Then Exit For
    Next I
    Skipall:
    GetFileExt = TMP$
End Function

Function GetFileName$ (FName$)

    $If WINDOWS Then
        SLASH$ = "\"
    $Else
            SLASH$ = "/"
    $End If

    TMP$ = ""
    T$ = GetFileExt(FName$)
    CUT = Len(T$)
    L = Len(FName$)
    TName$ = Left$(FName$, L - CUT)
    If InStr(1, TName$, SLASH$) = 0 Then
        TMP$ = TName$
        GoTo Skiploop
    End If
    For I = Len(TName$) To 1 Step -1
        C$ = Mid$(TName$, I, 1)
        If C$ <> SLASH$ Then
            TMP$ = C$ + TMP$
        Else
            Exit For
        End If
    Next I
    Skiploop:
    GetFileName = TMP$
End Function

Function GetFilePath$ (FName$)
    t1$ = GetFileExt(FName$)
    t2$ = GetFileName(FName$)
    L = Len(FName$) - (Len(t1$) + Len(t2$))
    GetFilePath$ = Left$(FName$, L)
End Function

Function IsNum%% (PassedText As String)
    text$ = PassedText
    special$ = UCase$(Left$(text$, 2))
    Select Case special$
        Case "&H", "&B", "&O"
            'check for symbols on right side of value
            r3$ = Right$(text$, 3)
            Select Case r3$
                Case "~&&", "~%%", "~%&" 'unsigned int64, unsigned byte, unsigned offset
                    text$ = Left$(text$, Len(text$) - 3)
                Case Else
                    r2$ = Right$(text$, 2)
                    Select Case r2$
                        Case "~&", "##", "%&", "%%", "~%", "&&" 'unsigned long, float, offset, byte, unsigned integer, int64
                            text$ = Left$(text$, Len(text$) - 2)
                        Case Else
                            r$ = Right$(text$, 1)
                            Select Case r$
                                Case "&", "#", "%", "!" 'long, double, integer, single
                                    text$ = Left$(text$, Len(text$) - 1)
                            End Select
                    End Select
            End Select
            check$ = "0123456789ABCDEF"
            If special$ = "&O" Then check$ = "01234567"
            If special$ = "&B" Then check$ = "01"
            temp$ = Mid$(UCase$(text$), 2)
            For i = 1 To Len(temp$)
                If InStr(check$, Mid$(temp$, i, 1)) = 0 Then Exit For
            Next
            If i <= Len(temp$) Then IsNum = -1
        Case Else
            If _Trim$(Str$(Val(text$))) = text$ Then IsNum = -1
    End Select
End Function



